@@ -16,6 +16,7 @@
 #include <unistd.h>
 #include "qiv.h"
 #include "xmalloc.h"
+#include "qiv_icon.h"
 
 static void setup_win(qiv_image *);
 //static void setup_magnify(qiv_image *, qiv_mgl *); // [lc]
@@ -24,79 +25,126 @@ static struct timeval load_before, load_after;
 static double load_elapsed;
 static GdkCursor *cursor, *visible_cursor, *invisible_cursor;
 
-#ifdef HAVE_EXIF
-//////////// these taken from gwenview/src/imageutils/ //////////////////
-//
-/* Explanation extracted from http://sylvana.net/jpegcrop/exif_orientation.html
+Imlib_Image im_from_pixbuf_loader(char * image_name, int * has_alpha)
+{
+  GError    *error=NULL;
+  GdkPixbuf *pixbuf_ori;
+  GdkPixbuf *pixbuf;
+  char * argbdata;
+  guchar * pixels;
+  guchar * pixels_ori;
+  int i,j,k,rs;
+  int pb_w, pb_h;
+  Imlib_Image * im=NULL;
+  const gchar *gdk_orientation = NULL;
 
-   For convenience, here is what the letter F would look like if it were tagged
-correctly and displayed by a program that ignores the orientation tag (thus
-showing the stored image):
-
-  1        2       3      4         5            6           7          8
-
-888888  888888      88  88      8888888888  88                  88  8888888888
-88          88      88  88      88  88      88  88          88  88      88  88
-8888      8888    8888  8888    88          8888888888  8888888888          88
-88          88      88  88
-88          88  888888  888888
-
-NORMAL  HFLIP   ROT_180 VFLIP   TRANSPOSE   ROT_90      TRANSVERSE  ROT_270
-*/
-
-enum Orientation {
-    NOT_AVAILABLE=0,
-    NORMAL  =1,
-    HFLIP   =2,
-    ROT_180 =3,
-    VFLIP   =4,
-    TRANSPOSE   =5,
-    ROT_90  =6,
-    TRANSVERSE  =7,
-    ROT_270 =8
-};
-
-// Exif
-#include "libexif/exif-data.h"
-
-#define flipH(q)    imlib_image_flip_horizontal();
-#define flipV(q)    imlib_image_flip_vertical();
-#define transpose(q) imlib_image_flip_diagonal();
-#define rot90(q)    imlib_image_orientate(1);
-#define rot180(q)   imlib_image_orientate(2);
-#define rot270(q)   imlib_image_orientate(3);
-#define swapWH(q)  swap(&q->orig_w, &q->orig_h); swap(&q->win_w, &q->win_h);
-void transform( qiv_image *q, enum Orientation orientation) {
-    switch (orientation) {
-     default: return;
-     case HFLIP:     flipH(q); snprintf(infotext, sizeof infotext, "(Flipped horizontally)"); break;
-     case VFLIP:     flipV(q); snprintf(infotext, sizeof infotext, "(Flipped vertically)"); break;
-     case ROT_180:   rot180(q); snprintf(infotext, sizeof infotext, "(Turned upside down)"); break;
-      case TRANSPOSE: transpose(q); swapWH(q); snprintf(infotext, sizeof infotext, "(Transposed)"); break;
-     case ROT_90:    rot90(q); swapWH(q); snprintf(infotext, sizeof infotext, "(Rotated left)"); break;
-     case TRANSVERSE: transpose(q); rot180(q); swapWH(q); snprintf(infotext, sizeof infotext, "(Transversed)"); break;
-     case ROT_270:   rot270(q); swapWH(q); snprintf(infotext, sizeof infotext, "(Rotated left)"); break;
+  pixbuf_ori = gdk_pixbuf_new_from_file(image_name, &error);
+  if (error != NULL)
+  {
+    /* Report error to user, and free error */
+    fprintf(stderr, "Unable to read file: %s\n", error->message);
+    g_error_free (error);
+  }
+  else
+  {
+#if GDK_PIXBUF_MINOR >= 12
+  if(autorotate)
+    {
+      gdk_orientation = gdk_pixbuf_get_option(pixbuf_ori, "orientation");
+      if(gdk_orientation)
+      {
+#ifdef DEBUG
+        printf("orientation %s\n", gdk_orientation);
+#endif
+        pixbuf = gdk_pixbuf_apply_embedded_orientation (pixbuf_ori);
+        g_object_unref(pixbuf_ori);
+        pixbuf_ori = pixbuf;
+      }
     }
-}
+#else
+#warning autoration needs at least gdk version 2.12
+#endif
 
-//#include "libexif/exif-tag.h"   //EXIF_TAG_ORIENTATION
-enum Orientation orient( const char * path) {
-    enum Orientation orientation = NOT_AVAILABLE;
+    *has_alpha =  ( gdk_pixbuf_get_n_channels(pixbuf_ori) == 4) ? 1 : 0;
 
-    ExifData * mExifData = exif_data_new_from_file( path);
-    if (mExifData) {
-        ExifEntry * mOrientationEntry = exif_content_get_entry( mExifData->ifd[ EXIF_IFD_0], EXIF_TAG_ORIENTATION);
-        if (mOrientationEntry) {
-            ExifByteOrder mByteOrder = exif_data_get_byte_order( mExifData);
-            short value=exif_get_short( mOrientationEntry->data, mByteOrder);
-            if (value>=NORMAL && value<=ROT_270)
-                orientation = value;    //Orientation( value);
+    pixels_ori = gdk_pixbuf_get_pixels(pixbuf_ori);
+    /* create checkboard if image has transparency */
+    if(*has_alpha)
+    {
+      pixbuf = gdk_pixbuf_composite_color_simple(pixbuf_ori, gdk_pixbuf_get_width(pixbuf_ori),
+                   gdk_pixbuf_get_height(pixbuf_ori), GDK_INTERP_NEAREST, 255, 0x08, 0x00666666, 0x00aaaaaa);
+
+      pixels = gdk_pixbuf_get_pixels(pixbuf);
+    }
+    else
+    {
+      pixbuf = pixbuf_ori;
+      pixels = pixels_ori;
+    }
+
+#ifdef DEBUG
+    printf("channels %i\n", gdk_pixbuf_get_n_channels(pixbuf));
+    printf("rowstride %i\n", gdk_pixbuf_get_rowstride(pixbuf));
+#endif
+
+    pb_w = gdk_pixbuf_get_width(pixbuf);
+    pb_h = gdk_pixbuf_get_height(pixbuf);
+
+    argbdata=malloc(4 * pb_w * pb_h);
+
+    /* create imlib2 compatible data */
+    if(*has_alpha)
+    {
+      for(i=0; i< pb_w*pb_h*4; i=i+4)
+      {
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+        argbdata[i+0]=pixels[i+2]; // B
+        argbdata[i+1]=pixels[i+1]; // G
+        argbdata[i+2]=pixels[i];   // R
+        // keep old alpha values
+        argbdata[i+3]=pixels_ori[i+3]; // Alpha
+#else
+        // BIG_ENDIAN
+        argbdata[i+3]=pixels[i+2]; // B
+        argbdata[i+2]=pixels[i+1]; // G
+        argbdata[i+1]=pixels[i];   // R
+        // keep old alpha values
+        argbdata[i+0]=pixels_ori[i+3]; // Alpha
+#endif
+      }
+    }
+    else
+    {
+      rs = gdk_pixbuf_get_rowstride (pixbuf);
+      k=0;
+      for(i=0; i < pb_h; i++)
+      {
+        for(j=0; j< pb_w*3; j+=3)
+        {
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+          argbdata[k++]=pixels[i*rs+j+2]; // B
+          argbdata[k++]=pixels[i*rs+j+1]; // G
+          argbdata[k++]=pixels[i*rs+j];   // R
+          argbdata[k++]=0;
+#else
+          argbdata[k++]=0;
+          argbdata[k++]=pixels[i*rs+j];   // R
+          argbdata[k++]=pixels[i*rs+j+1]; // G
+          argbdata[k++]=pixels[i*rs+j+2]; // B
+#endif
         }
-        exif_data_unref( mExifData );
+      }
     }
-    return orientation;
+    im = imlib_create_image_using_copied_data(pb_w, pb_h, (DATA32*)argbdata);
+    free(argbdata);
+    if(*has_alpha)
+    {
+      g_object_unref(pixbuf);
+    }
+    g_object_unref(pixbuf_ori);
+  }
+  return im;
 }
-#endif  //HAVE_EXIF
 
 /*
  *    Load & display image
@@ -106,6 +154,8 @@ void qiv_load_image(qiv_image *q)
 {
   struct stat statbuf;
   const char * image_name = image_names[ image_idx];
+  Imlib_Image * im=NULL;
+  int has_alpha=0;
 
   gettimeofday(&load_before, 0);
 
@@ -114,7 +164,16 @@ void qiv_load_image(qiv_image *q)
 
   stat(image_name, &statbuf);
   current_mtime = statbuf.st_mtime;
-  Imlib_Image * im = imlib_load_image( (char*)image_name );
+
+#ifdef DEBUG
+  g_print("loading %s\n",image_name);
+#endif
+
+  /* use gdk_pixbuf for loading
+  im = imlib_load_image( (char*)image_name );
+  */
+
+  im = im_from_pixbuf_loader( (char*)image_name, &has_alpha);
 
   if (!im) { /* error */
     q->error = 1;
@@ -122,18 +181,28 @@ void qiv_load_image(qiv_image *q)
     q->orig_h = 300;
   } else { /* Retrieve image properties */
     imlib_context_set_image(im);
+    if(has_alpha) {
+      imlib_image_set_has_alpha(has_alpha);
+    }
     q->error = 0;
     q->orig_w = imlib_image_get_width();
     q->orig_h = imlib_image_get_height();
-#ifdef HAVE_EXIF
-    if (autorotate) {
-      transform( q, orient( image_name));
+  }
+
+  if (rotation>0) {
+    imlib_image_orientate(rotation);
+    if (rotation!=2) {
+      swap(&q->orig_w, &q->orig_h);
+      swap(&q->win_w, &q->win_h);
     }
-#endif
   }
 
   check_size(q, TRUE);
 
+  if (rotation>0) {
+    correct_image_position(q);
+  }
+
   if (first) {
     setup_win(q);
     first = 0;
@@ -146,10 +215,7 @@ void qiv_load_image(qiv_image *q)
       qiv_exit(1);
     }
     set_desktop_image(q);
-    if(slide)
-      return;
-    else
-      qiv_exit(0);
+    qiv_exit(0);
   }
 
   gdk_window_set_background(q->win, im ? &image_bg : &error_bg);
@@ -210,6 +276,9 @@ static void setup_win(qiv_image *q)
 {
   GdkWindowAttr attr;
   GdkPixmap *cursor_pixmap;
+  static GdkPixbuf *icon=NULL;;
+  static GdkPixmap *icon_pixmap;
+  static GdkBitmap *icon_bitmap;
 
   if (!fullscreen) {
     attr.window_type=GDK_WINDOW_TOPLEVEL;
@@ -221,6 +290,14 @@ static void setup_win(qiv_image *q)
     attr.height = q->win_h;
     q->win = gdk_window_new(NULL, &attr, GDK_WA_X|GDK_WA_Y);
 
+    /* create the icon only once */
+    if(icon==NULL)
+    {
+      icon = gdk_pixbuf_new_from_inline (-1, qiv_icon, FALSE, NULL);
+      gdk_pixbuf_render_pixmap_and_mask(icon, &icon_pixmap, &icon_bitmap, 10);
+    }
+    gdk_window_set_icon(q->win, NULL, icon_pixmap, icon_bitmap);
+                                             
     if (center) {
       GdkGeometry geometry = {
         .min_width = q->win_w,
@@ -243,7 +320,7 @@ static void setup_win(qiv_image *q)
         GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE);
       gdk_window_resize(q->win, q->win_w, q->win_h);
     }
-
+   if (!(to_root || to_root_t || to_root_s))
     gdk_window_show(q->win);
 
   } else { /* fullscreen */
@@ -256,7 +333,8 @@ static void setup_win(qiv_image *q)
     attr.height=screen_y;
     q->win = gdk_window_new(NULL, &attr, GDK_WA_X|GDK_WA_Y);
     gdk_window_set_cursor(q->win, cursor);
-    gdk_window_show(q->win);
+    if (!(to_root || to_root_t || to_root_s))
+      gdk_window_show(q->win);
   }
 
   q->bg_gc = gdk_gc_new(q->win);
@@ -440,9 +518,13 @@ void zoom_maxpect(qiv_image *q)
 
 void reload_image(qiv_image *q)
 {
+  Imlib_Image *im;
+  int has_alpha = 0;
+
   imlib_image_set_changes_on_disk();
 
-  Imlib_Image *im = imlib_load_image(image_names[image_idx]);
+  im = im_from_pixbuf_loader(image_names[image_idx], &has_alpha);
+
   if (!im && watch_file)
     return;
 
@@ -461,6 +543,9 @@ void reload_image(qiv_image *q)
   } else { /* Retrieve image properties */
     q->error = 0;
     imlib_context_set_image(im);
+    if(has_alpha) {
+      imlib_image_set_has_alpha(has_alpha);
+    }
     q->orig_w = imlib_image_get_width();
     q->orig_h = imlib_image_get_height();
   }
@@ -468,7 +553,7 @@ void reload_image(qiv_image *q)
   q->win_w = (gint)(q->orig_w * (1 + zoom_factor * 0.1));
   q->win_h = (gint)(q->orig_h * (1 + zoom_factor * 0.1));
   reset_mod(q);
-  center_image(q);
+  if (center) center_image(q);
 }
 
 void check_size(qiv_image *q, gint reset)
