@@ -23,7 +23,7 @@
 extern char *optarg;
 extern int optind, opterr, optopt;
 
-static char *short_options = "ab:c:d:efg:hilmno:pq:rstuvw:xyzA:BDF:GIMNPRSTW:X:";
+static char *short_options = "ab:c:Cd:efg:hilmno:pq:rstuvw:xyzA:BDF:GIMNPRSTW:X:";
 static struct option long_options[] =
 {
     {"do_grab",          0, NULL, 'a'},
@@ -54,6 +54,7 @@ static struct option long_options[] =
     {"root_s",           0, NULL, 'z'},
     {"select_dir",       1, NULL, 'A'},
     {"browse",           0, NULL, 'B'},
+    {"cycle",            0, NULL, 'C'},
     {"no_sort",          0, NULL, 'D'},
     {"file",             1, NULL, 'F'},
     {"disable_grab",     0, NULL, 'G'},
@@ -273,6 +274,8 @@ void options_read(int argc, char **argv, qiv_image *q)
                 break;
             case 'B': browse=1;
                 break;
+            case 'C': cycle=1;
+                break;
             case 'D': need_sort = 0;
                 break;
             case 'F': if(rreadfile(optarg) < 0) {
@@ -349,18 +352,17 @@ void options_read(int argc, char **argv, qiv_image *q)
         }
     }
 
-    if (need_sort) {
-        if (browse) {
-            char *tmp = (char *)xmalloc(strlen(image_names[0])+1);
-            strcpy(tmp,image_names[0]);
-            rreaddir(dirname(image_names[0]),0);
-            qsort(image_names, images, sizeof *image_names, my_strcmp);
-            image_idx = find_image(images,image_names,tmp);
-            free(tmp);
-        } else {
+    if (browse) {
+        images = 0; /* avoid displaying same filename twice */
+        char *tmp = (char *)xmalloc(strlen(image_names[0])+1);
+        strcpy(tmp,image_names[0]);
+        rreaddir(dirname(image_names[0]),0);
+        if (need_sort) {
             qsort(image_names, images, sizeof *image_names, my_strcmp);
         }
-    } else if (browse) {
-        rreaddir(dirname(image_names[0]),0);
+        image_idx = find_image(images,image_names,tmp);
+        free(tmp);
+    } else if (need_sort) {
+        qsort(image_names, images, sizeof *image_names, my_strcmp);
     }
 }
