@@ -429,8 +429,9 @@ void next_image(int direction)
     image_idx = (image_idx + direction) % images;
     if (image_idx < 0)
       image_idx += images;
+    else if (cycle && image_idx == 0)
+      qiv_exit(0);
   }
-
 }
 
 int checked_atoi (const char *s)
@@ -472,6 +473,7 @@ void show_help(char *name, int exit_status)
           "    --browse, -B           Scan directory of file for browsing\n"
           "    --center, -e           Disable window centering\n"
           "    --contrast, -c x       Set contrast to x (-32..32)\n"
+          "    --cycle, -C            do not cycle after last image\n"
           "    --display x            Open qiv window on display x\n"
           "    --do_grab, -a          Grab the pointer in windowed mode\n"
           "    --disable_grab, -G     Disable pointer/kbd grab in fullscreen mode\n"
@@ -497,8 +499,13 @@ void show_help(char *name, int exit_status)
           "    --watch, -T            Reload the image if it has changed on disk\n"
           "    --recursivedir, -u     Recursively include all files\n"
           "    --select_dir, -A x     Store the selected files in dir x (default is .qiv-select)\n"
+#if GDK_PIXBUF_MINOR >= 12
           "    --autorotate, -l       Autorotate JPEGs according to EXIF rotation tag\n"
+#endif
+          "    --rotate, -q x         Rotate 90(x=1),180(x=2),270(x=3) degrees clockwise\n"
+#ifdef GTD_XINERAMA
           "    --xineramascreen, -X x Use screen x as preferred Xinerama screen\n"
+#endif
           "    --version, -v          Print version information and exit\n"
           "\n"
           "Slideshow options:\n"
@@ -585,20 +592,22 @@ int rreaddir(const char *dirname, int recursive)
         strcmp(entry->d_name,TRASH_DIR) == 0)
       continue;
     snprintf(name, sizeof name, "%s/%s", cdirname, entry->d_name);
-    if (stat(name, &sb) >= 0 && S_ISDIR(sb.st_mode)) {
-      if (!recursive)
-        continue;
-      rreaddir(name,1);
-    }
-    else {
-      if (images >= max_image_cnt) {
-        max_image_cnt += 8192;
-        if (!image_names)
-          image_names = (char**)xmalloc(max_image_cnt * sizeof(char*));
-        else
-          image_names = (char**)xrealloc(image_names,max_image_cnt*sizeof(char*));
+    if (lstat(name, &sb) >= 0) {
+      if (S_ISDIR(sb.st_mode)) {
+        if (!recursive)
+          continue;
+        rreaddir(name,1);
+      }
+      else {
+        if (images >= max_image_cnt) {
+          max_image_cnt += 8192;
+          if (!image_names)
+            image_names = (char**)xmalloc(max_image_cnt * sizeof(char*));
+          else
+            image_names = (char**)xrealloc(image_names,max_image_cnt*sizeof(char*));
+        }
+        image_names[images++] = strdup(name);
       }
-      image_names[images++] = strdup(name);
     }
   }
   closedir(d);
